﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;



namespace I2CTempSensor
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private const byte TEMP_TMP102_ADDR = 0x48;
        private I2cDevice I2CTemp;
        private const byte TempRegisterAddr = 0x00;
        private const byte ConfigRegisterAddr = 0x01;
        private const byte TlowRegisterAddr = 0x02;
        private const byte ThighRegisterAddr = 0x03;



#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public MainWindow()
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            InitializeComponent();
            TempSetup();
        }

        private async void TempSetup()
        {
            I2cConnectionSettings i2cSettings = new I2cConnectionSettings(TEMP_TMP102_ADDR);
            i2cSettings.BusSpeed = I2cBusSpeed.FastMode;

            string aqs = I2cDevice.GetDeviceSelector("I2C1");
            DeviceInformationCollection i2cDevInfo = await DeviceInformation.FindAllAsync(aqs);
            I2CTemp = await I2cDevice.FromIdAsync(i2cDevInfo[0].Id, i2cSettings);
            if (I2CTemp == null)
            {
                txtStatus.Text = "Error getting I2C connection";
            }
            else
            {
                txtStatus.Text = "Ready to Read Temp";
            }
        }

        private void btnReadTemp_Click(object sender, RoutedEventArgs e)
        {
            byte[] Readbuf = new byte[2];
            byte[] RegAddrBuf = new byte[] { TempRegisterAddr };
            I2CTemp.WriteRead(RegAddrBuf, Readbuf);


            byte tempMSB = Readbuf[0];
            var tempMSBs = tempMSB << 4;
            byte tempLSB = Readbuf[1];
            var tempLSBs = tempLSB >> 4;
            var tempFinal = tempMSBs + tempLSBs;

            if ((tempFinal & 0x800) == 0x800) //detect if negative
            {
                tempFinal = (tempFinal ^ 0xFFF) + 1; //twos compliment
                tempFinal = -tempFinal;
            }

            double tempC = Convert.ToDouble(tempFinal) * 0.0625;
            double tempF = (tempC * 1.8) + 32.0;

            txtTempCReading.Text = tempC.ToString() + " C";
            txtTempFReading.Text = tempF.ToString() + " F";
        }
    }
}
